;**************************************************************************
;                                EJEMPLO 3
;                          Los Convertidores A/D
;                          Autor: Jorge R. Muoz                                                    
;**************************************************************************
; El siguiente ejemplo es una aplicacin con C PIC 16C71 de la casa 
; Microchip.
; La aplicacin consiste en implementar dos convertidores A/D,(RA0-AN0 y 
; RA1-AN1),cuyas salidas sern reflejadas en RB<0-7>(leds) seleccionables 
; mediante un interruptor en RA2.
; La frecuencia de trabajo es de 4MHz cuyo periodo es de 250ns.
;**************************************************************************
;
        list P =16C71            ; C PIC16C71
        list C =132              ; Listado a 132 caracteres.
;
OPCION  equ  01h                 ; Registro de opciones.
STATUS  equ  03h                 ; Registro de estado.
PORT_A  equ  05h                 ; Puerta A.
PORT_B  equ  06h                 ; Puerta B.
TRIS_A  equ  05h                 ; Reg. de programacin de PA.
TRIS_B  equ  06h                 ; Reg. de programacin de PB.
ADCON0  equ  08h                 ; Reg. de programacin y control del ADC.
ADCON1  equ  08h                 ; Reg. de programacin del ADC.
ADDRES  equ  09h                 ; Reg. de almacenamiento de conversin.
INTCON  equ  0Bh                 ; Reg. de control de interrupciones.
CONTA   equ  0Ch                 ; Variable de Temporizacin.
;
;**************************************************************************
;


                org 0            ; Vector de Reset.
                goto INICIO      ; Salto al inicio.
;
                org 5            ; Direccin de inicio.
;
;********************** Programa principal ********************************
;
;                                
INICIO          bsf STATUS,5     ; Seleccin del Banco 1.

                clrf TRIS_B      ; RB<7-0> salida de conversores A/D.
;
                movlw B'00000111'; RA0,RA1 y RA2 como entradas del 
                movwf TRIS_A     ; interruptor y del conversor A/D.
;
                movlw B'00000010'; Configuracin de pines del ADCON1 como 
                movwf ADCON1     ; entradas.RA0 y RA1=Analgicas,RA2 Digital
                                 ; y Vref=Vdd.
;
                movlw B'10000000'; Programacion de Reg. de Opciones
                movwf OPCION

                movlw B'00000000'; Desactivar interrupciones.
                movwf INTCON     ;
;                
                bcf STATUS,5     ; Seleccin del Banco 0.
;
;******************** Rutina de eleccin del conversor ********************
;                
QUIEN_ES        btfss PORT_A,2   ; Es el bit RA2=1 (E digital).
                goto ES_AN_0     ; NO.
;
;******************** Rutina de configuracin del conversor ***************
;
                movlw B'10001001'; SI.Activar conversor,borrar ADIF,GO      
                movwf ADCON0     ; (ADCON0<2>),canal RA1/AN1 y Fosc/32.
                goto NO_FIN

ES_AN_0         movlw B'10000001'; Activar conversor,borrar ADIF,GO
                movwf ADCON0     ; (ADCON0<2>),canal RA0/AN0 y Fosc/32.
;
;******************** Rutina de espera de fin de conversin **************
;
NO_FIN          movlw B'00001010'
                movwf CONTA      ; Carga de contador.
DELAY           decfsz CONTA,1
                goto DELAY
                bsf ADCON0,2     ; Activar Convertidor GO.
NO_FIN_1        btfsc ADCON0,2   ; Es DONE(ADCON0<2>)=0(Fin de conversin)?
                goto NO_FIN_1    ; NO.Siga esperando el fin de conversin.
;
;******************** Rutina de muestreo en PB ****************************
;
                movf ADDRES,0    ; SI.ADRES==>W Recoge resultado de la                                                     
                movwf PORT_B     ; conversin y saca en RB<0-7>.
                bcf ADCON0,1     ; Bajar flag ADIF.
                goto QUIEN_ES    ; Reiniciar el ciclo.
                
                end

